/****************************************************************************
**
** Copyright (C) 1992-2007 Trolltech ASA. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/


#include <qscriptengine.h>

#include <stdlib.h>

#include <QtCore/QFile>
#include <QtCore/QTextStream>
#include <QtGui/QApplication>

#include "testqs.h"
#include "uiutils.h"

static void interactive(QScriptEngine &eng)
{
    QTextStream qin(stdin, QFile::ReadOnly);

    const char *qscript_prompt = "qscript> ";
    const char *dot_prompt = ".... ";
    const char *prompt = qscript_prompt;

    QString code;
    int lineno = 0;

    forever {
        QString line;

        printf("%s", prompt);
        fflush(stdout);

        line = qin.readLine();
        if (line.isNull())
            break;

        code += line;
        code += QLatin1Char('\n');

        if (line.trimmed().isEmpty())
            continue;

        else if (! eng.canEvaluate(code))
            prompt = dot_prompt;

        else {
            QScriptValue result = eng.evaluate(code);

            code.clear();
            prompt = qscript_prompt;

            if (! result.isUndefined())
                printf("%s\n", qPrintable(result.toString()));
        }

        ++lineno;
    }
}

Q_SCRIPT_DECLARE_QCLASS( TestQs, QObject* );

int main(int argc, char *argv[])
{
    QApplication app( argc, argv );

    QScriptEngine eng;
    QScriptValue globalObject = eng.globalObject();

    UiUtils::install( &eng );

    if (! *++argv) {
        interactive(eng);
        return EXIT_SUCCESS;
    }

    while (const char *arg = *argv++) {
        QString fn = QString::fromLocal8Bit(arg);

        if (fn == QLatin1String("-i")) {
            interactive(eng);
            break;
        }

        QString contents;

        if (fn == QLatin1String("-")) {
            QTextStream stream(stdin, QFile::ReadOnly);
            contents = stream.readAll();
        }

        else {
            QFile file(fn);

            if (file.open(QFile::ReadOnly)) {
                QTextStream stream(&file);
                contents = stream.readAll();
                file.close();
            }
        }

        if (contents.isEmpty())
            continue;

        QScriptValue r = eng.evaluate(contents);
        if (eng.uncaughtException()) {
            int line = eng.uncaughtExceptionLineNumber();
            fprintf (stderr, "*** %d: %s\n\t%s\n\n", line, qPrintable(fn), qPrintable(r.toString()));
            return EXIT_FAILURE;
        }
    }

    return EXIT_SUCCESS;
}
